import bpy
from ... import Utility_Functions
import os
from ...Tags_Utils import Tags


class VIEWFINDER_OT_Add_Filter_Tags(bpy.types.Operator):
    """Filter Tags"""

    bl_idname = "viewfinder.add_filter_tags"
    bl_label = "Add Filter Tags"
    bl_options = {"UNDO", "REGISTER"}

    tag: bpy.props.StringProperty()

    def execute(self, context):
        if context.scene.Viewfinder_Scene_Properties.FILTER_Tags == "":
            context.scene.Viewfinder_Scene_Properties.FILTER_Tags = self.tag
        else:
            filter_tags = context.scene.Viewfinder_Scene_Properties.FILTER_Tags
            tags = Tags(filter_tags)
            tags.add_tag(self.tag)
            context.scene.Viewfinder_Scene_Properties.FILTER_Tags = tags.to_string()

        Utility_Functions.update_UI()
        return {"FINISHED"}


class VIEWFINDER_OT_Remove_Filter_Tags(bpy.types.Operator):
    """Remove Filter Tags"""

    bl_idname = "viewfinder.remove_filter_tags"
    bl_label = "Remove Filter Tags"
    bl_options = {"UNDO", "REGISTER"}

    tag: bpy.props.StringProperty()

    def execute(self, context):
        filter_tags = context.scene.Viewfinder_Scene_Properties.FILTER_Tags
        tags = Tags(filter_tags)
        tags.remove_tag(self.tag)
        context.scene.Viewfinder_Scene_Properties.FILTER_Tags = tags.to_string()

        Utility_Functions.update_UI()
        return {"FINISHED"}


classes = [VIEWFINDER_OT_Add_Filter_Tags, VIEWFINDER_OT_Remove_Filter_Tags]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


if __name__ == "__main__":
    register()
