import bpy
from ... import Utility_Functions
import os


class VIEWFINDER_List_Select_Target(bpy.types.Operator):
    """List Select Target"""

    bl_idname = "viewfinder.list_select_target"
    bl_label = "List Select Target"
    bl_options = {"UNDO", "REGISTER"}

    object: bpy.props.StringProperty()

    @classmethod
    def poll(cls, context):
        return context.mode in ["OBJECT"]

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

    def draw(self, context):
        layout = self.layout

        object = context.view_layer.objects.get(self.object)
        constraints = [
            constraint for constraint in object.constraints if constraint.target
        ]

        for constraint in constraints:
            target = constraint.target
            if target:
                row = layout.row(align=True)
                op = row.operator(
                    "viewfinder.select_object", text="", icon="RESTRICT_SELECT_OFF"
                )
                op.object = target.name
                row.label(text=target.name)
                row.label(text=constraint.type, icon="CONSTRAINT")

    def execute(self, context):
        Utility_Functions.update_UI()
        return {"FINISHED"}


class VIEWFINDER_Select_Object(bpy.types.Operator):
    """Select Object"""

    bl_idname = "viewfinder.select_object"
    bl_label = "Select Object"
    bl_options = {"UNDO", "REGISTER"}

    object: bpy.props.StringProperty()
    deselect: bpy.props.BoolProperty()

    @classmethod
    def poll(cls, context):
        return context.mode in ["OBJECT"]

    def invoke(self, context, event):
        if event.shift:
            self.deselect = False
        else:
            self.deselect = True

        return self.execute(context)

    def execute(self, context):
        if self.deselect:
            for obj in bpy.data.objects:
                obj.select_set(False)

        object = context.view_layer.objects.get(self.object)
        if object:
            object.select_set(True)
            context.view_layer.objects.active = object

        Utility_Functions.update_UI()
        return {"FINISHED"}


class VIEWFINDER_Select_Target(bpy.types.Operator):
    """Select Target"""

    bl_idname = "viewfinder.select_target"
    bl_label = "Select Target"
    bl_options = {"UNDO", "REGISTER"}

    object: bpy.props.StringProperty()

    @classmethod
    def poll(cls, context):
        return context.mode in ["OBJECT"]

    def execute(self, context):
        object = context.view_layer.objects.get(self.object)

        bpy.ops.object.select_all(action="DESELECT")

        if object:
            for constraint in object.constraints:
                if hasattr(constraint, "target"):
                    target = constraint.target
                    if target:
                        target.select_set(True)
                        context.view_layer.objects.active = target

        Utility_Functions.update_UI()
        return {"FINISHED"}


classes = [
    VIEWFINDER_Select_Target,
    VIEWFINDER_Select_Object,
    VIEWFINDER_List_Select_Target,
]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


if __name__ == "__main__":
    register()
