import bpy
import os
from .. import Utility_Functions
from ..Tags_Utils import Tags


class VIEWFINDER_PT_Camera_List_Panel(bpy.types.Panel):
    """Viewfinder Camera List"""

    bl_label = "Viewfinder Camera List"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Viewfinder"

    @classmethod
    def poll(cls, context):
        preferences = Utility_Functions.get_addon_preferences()
        return preferences.PANEL_Camera_List_Panel

    # def draw_icon_expose(self, context, layout):
    #     preferences = Utility_Functions.get_addon_preferences()
    #
    #     if Utility_Functions.draw_subpanel(
    #         preferences,
    #         preferences.SHOW_Camera_List_Icon_Expose,
    #         "SHOW_Camera_List_Icon_Expose",
    #         "Icon Expose",
    #         layout,
    #     ):
    #         layout.prop(preferences, "ICON_Camera_List_Reorder", text="Reorder")
    #         layout.prop(preferences, "ICON_Camera_List_Pin", text="Pin")
    #         layout.prop(preferences, "ICON_Camera_List_Priority", text="Priority")
    #         layout.prop(
    #             preferences, "ICON_Camera_List_Select_Camera", text="Select Camera"
    #         )
    #
    #         layout.prop(
    #             preferences, "ICON_Camera_List_Select_Target", text="Select Target"
    #         )
    #
    #         layout.prop(preferences, "ICON_Camera_List_View_Camera", text="View Camera")
    #         layout.prop(preferences, "ICON_Camera_List_Set_Marker", text="Set Marker")
    #         layout.prop(preferences, "ICON_Camera_List_Resolution", text="Resolution")
    #
    #         layout.prop(preferences, "ICON_Camera_List_Find_Camera", text="Find Camera")
    #         layout.prop(
    #             preferences,
    #             "ICON_Camera_List_Duplicate_Camera",
    #             text="Duplicate Camera",
    #         )
    #         layout.prop(preferences, "ICON_Camera_List_Tags", text="Tags")
    #         layout.prop(
    #             preferences, "ICON_Camera_List_Remove_Camera", text="Remove Camera"
    #         )

    def draw(self, context):
        layout = self.layout
        scn = context.scene
        scn_properties = scn.Viewfinder_Scene_Properties
        col = layout.column(align=True)

        cameras = self.get_cameras(context)

        object_tags = []

        for camera in cameras:
            object_tags += Tags(camera.Viewfinder_Object_Properties.Tags).taglist

        object_tags = list(dict.fromkeys(object_tags))
        filter_tags = Tags(scn_properties.FILTER_Tags)

        preferences = Utility_Functions.get_addon_preferences()

        split = col.split(factor=0.7, align=True)
        split.prop(scn_properties, "FILTER_Name", text="", icon="VIEWZOOM")
        split.prop(scn_properties, "FILTER_Camera_List", text="", icon="FILTER")

        if preferences.Show_Tags_Filter:
            col.separator()
            split = col.split(factor=0.7, align=True)

            split.prop(
                scn_properties,
                "FILTER_Tags",
                text="",
                placeholder="Tags",
                icon="BOOKMARKS",
            )
            split.prop(scn_properties, "Tag_Mode", icon="FILTER", text="")
            # split.prop(
            #     scn_properties,
            #     "SHOW_Tags",
            #     text="Available",
            #     icon="BOOKMARKS",
            # )

            if len(object_tags) > 0:
                box = col.box()

                Show_Available_Tags = Utility_Functions.draw_subpanel(
                    scn_properties,
                    scn_properties.SHOW_Tags,
                    "SHOW_Tags",
                    "Show Available Tags",
                    box,
                )
                if scn_properties.SHOW_Tags:
                    row_box = box.row(align=True)

                    col_row = row_box.column()
                    if scn_properties.tag_variable_width:
                        col_row.enabled = False
                    else:
                        col_row.enabled = True
                    col_row.prop(scn_properties, "tag_column", text="Tag Column")
                    col_row = row_box.column()

                    #     tags = set()
                    #
                    #     if scn_properties.Tag_Mode == "EXCLUSIVE":
                    #         for camera in cameras:
                    #             tag_list = [
                    #                 tag.strip().lower()
                    #                 for tag in camera.Viewfinder_Object_Properties.Tags.split(
                    #                     ","
                    #                 )
                    #             ]
                    #             for tag in tag_list:
                    #                 if tag == "":
                    #                     continue
                    #
                    #                 filter_tags = [
                    #                     fil_tag.strip().lower()
                    #                     for fil_tag in context.scene.Viewfinder_Scene_Properties.FILTER_Tags.strip().split(
                    #                         ","
                    #                     )
                    #                 ]
                    #                 filter_tags = list(dict.fromkeys(filter_tags))
                    #
                    #                 if tag in filter_tags:
                    #                     tags.add((tag, True))
                    #                 else:
                    #                     tags.add((tag, False))
                    #
                    #     if scn_properties.Tag_Mode == "INCLUSIVE":
                    #         for camera in bpy.data.objects:
                    #             if camera.type == "CAMERA":
                    #                 tag_list = [
                    #                     tag.strip().lower()
                    #                     for tag in camera.Viewfinder_Object_Properties.Tags.split(
                    #                         ","
                    #                     )
                    #                 ]
                    #                 for tag in tag_list:
                    #                     if tag == "":
                    #                         continue
                    #
                    #                     filter_tags = [
                    #                         fil_tag.strip().lower()
                    #                         for fil_tag in context.scene.Viewfinder_Scene_Properties.FILTER_Tags.strip().split(
                    #                             ","
                    #                         )
                    #                     ]
                    #                     filter_tags = list(dict.fromkeys(filter_tags))
                    #
                    #                     if tag in filter_tags:
                    #                         tags.add((tag, True))
                    #                     else:
                    #                         tags.add((tag, False))

                    ################################################

                    # box_row = col.grid_flow(row_major=True, columns=4)
                    #
                    # tags = set()
                    #

                    box_row = box.grid_flow(
                        row_major=True, columns=scn_properties.tag_column, align=True
                    )

                    if len(object_tags) > 0:
                        for tag in object_tags:
                            if filter_tags.include(tag):
                                op = box_row.operator(
                                    "viewfinder.remove_filter_tags",
                                    text=tag,
                                    icon="BOOKMARKS",
                                    depress=True,
                                )
                                op.tag = tag
                            else:
                                op = box_row.operator(
                                    "viewfinder.add_filter_tags",
                                    text=tag,
                                    icon="BOOKMARKS",
                                    depress=False,
                                )
                                op.tag = tag
                    else:
                        box_row.label(text="No Tags Found", icon="INFO")

            # t = Tags(tags)
            #
            # if len(t) > 0:
            #     b_row = box.row(align=True)
            #     b_row.prop(scn_properties, "Tag_Mode", expand=True)
            #     box_row = box.grid_flow(row_major=True, columns=4)
            #
            #     for tag in t:
            #         op = box_row.operator(
            #             "viewfinder.remove_filter_tags",
            #             text=tag,
            #             icon="BOOKMARKS",
            #         )
            #         op.tag = tag
            #         op = box_row.operator(
            #             "viewfinder.add_filter_tags",
            #             text=tag,
            #             icon="BOOKMARKS",
            #         )
            #         op.tag = tag

            ##############################################

            # for t, s in tags:
            #     if s:
            #         op = box_row.operator(
            #             "viewfinder.remove_filter_tags",
            #             text=tag,
            #             icon="BOOKMARKS",
            #             depress=s,
            #         )
            #         op.tag = tag
            #     else:
            #         op = box_row.operator(
            #             "viewfinder.add_filter_tags",
            #             text=tag,
            #             icon="BOOKMARKS",
            #             depress=s,
            #         )
            #         op.tag = tag
            # else:
            #     box_row.label(text="No Tags Found", icon="INFO")

        if scn_properties.FILTER_Camera_List == "SCENE":
            col.separator()
            col.prop(context.scene, "camera", text="Scene Camera")

        col.separator()
        # row = col.row(align=True)
        # row.prop(scn_properties, "Set_Resolution", text="Use Resolution")
        # col.separator()

        row = col.row(align=True)

        box = row.box()
        box_col = box.column(align=True)
        # if preferences.BTN_Toggle_All_Subpanel:
        #     col.operator("viewfinder.toggle_all_subpanel", text="Toggle All Subpanel")

        #     if context.scene.Viewfinder_Scene_Properties.Tag_Mode == "EXCLUSIVE":
        tag_mode = context.scene.Viewfinder_Scene_Properties.Tag_Mode

        if filter_tags.count() > 0:
            cameras = self.filter_cameras_by_tag(cameras, filter_tags, tag_mode)
        self.draw_camera_list(context, box_col, cameras)

        col_side = row.column(align=True)
        col_side.operator("viewfinder.toggle_all_subpanel", text="", icon="MENU_PANEL")
        col_side.separator()
        col_side.menu("VIEWFINDER_MT_viewfinder_icon_menu", text="", icon="VIS_SEL_11")
        col_side.separator()
        # operator = col_side.operator(
        #     "viewfinder.create_camera_from_view",
        #     text="",
        #     icon="ADD",
        # )
        col_side.prop(context.space_data, "lock_camera", text="", icon="LOCKED")
        col_side.separator()
        # menu(menu, text='', text_ctxt='', translate=True, icon='NONE', icon_value=0)

        # self.draw_icon_expose(context, layout)

    def filter_cameras_by_tag(self, cameras, filter_tags, mode):
        filtered_cameras = []
        filter = filter_tags

        # for camera in cameras:
        #     obj_tags = Tags(camera.Viewfinder_Object_Properties.Tags).taglist
        #     if any(tag for tag in obj_tags if tag in filter_tags):
        #         filtered_cameras.append(camera)
        for camera in cameras:
            if Tags(camera.Viewfinder_Object_Properties.Tags).match_tag_filter(
                filter, mode
            ):
                filtered_cameras.append(camera)

        return filtered_cameras

    def get_cameras(self, context):
        scn = context.scene
        scn_properties = scn.Viewfinder_Scene_Properties

        objects = []

        if scn_properties.FILTER_Camera_List in ["ALL"]:
            objects = [
                object
                for object in context.view_layer.objects
                if object.type == "CAMERA"
            ]

        if scn_properties.FILTER_Camera_List in ["SELECTED"]:
            objects = [
                object for object in context.selected_objects if object.type == "CAMERA"
            ]

        if scn_properties.FILTER_Camera_List in ["SCENE"]:
            if context.scene.camera:
                objects = [context.scene.camera]

        if scn_properties.FILTER_Camera_List in ["ACTIVE"]:
            if context.active_object:
                objects = [context.active_object]

        if not scn_properties.FILTER_Name == "":
            objects = [
                object
                for object in objects
                if scn_properties.FILTER_Name.lower() in object.name.lower()
            ]

            #####################################

        # if not context.scene.Viewfinder_Scene_Properties.FILTER_Tags == "":
        #     pass
        # box_row = col.grid_flow(row_major=True, columns=4)
        # object_tags = []
        #
        # for camera in cameras:
        #     object_tags += Tags(
        #         camera.Viewfinder_Object_Properties.Tags
        #     ).taglist
        #
        # object_tags = list(dict.fromkeys(object_tags))
        #
        # filter_tags = Tags(scn_properties.FILTER_Tags).taglist
        #
        # if len(object_tags) > 0:
        #     for tag in object_tags:
        #         if tag in filter_tags:
        #             op = box_row.operator(
        #                 "viewfinder.remove_filter_tags",
        #                 text=tag,
        #                 icon="BOOKMARKS",
        #                 depress=True,
        #             )
        #             op.tag = tag
        #         else:
        #             op = box_row.operator(
        #                 "viewfinder.add_filter_tags",
        #                 text=tag,
        #                 icon="BOOKMARKS",
        #                 depress=False,
        #             )
        #             op.tag = tag
        # else:
        #     box_row.label(text="No Tags Found", icon="INFO")

        #####################################

        # if not context.scene.Viewfinder_Scene_Properties.FILTER_Tags == "":
        #     filter_tags = [
        #         tag.strip().lower()
        #         for tag in context.scene.Viewfinder_Scene_Properties.FILTER_Tags.strip().split(
        #             ","
        #         )
        #     ]
        #     filter_tags = list(dict.fromkeys(filter_tags))
        #     tag_filter_objects = []
        #
        #     if context.scene.Viewfinder_Scene_Properties.Tag_Mode == "EXCLUSIVE":
        #         for object in objects:
        #             Obj_tags = [
        #                 tag.strip().lower()
        #                 for tag in object.Viewfinder_Object_Properties.Tags.strip().split(
        #                     ","
        #                 )
        #             ]
        #             Obj_tags = list(dict.fromkeys(Obj_tags))
        #
        #             if all(t in Obj_tags for t in filter_tags):
        #                 tag_filter_objects.append(object)
        #
        #     if context.scene.Viewfinder_Scene_Properties.Tag_Mode == "INCLUSIVE":
        #         for object in objects:
        #             Obj_tags = [
        #                 tag.strip().lower()
        #                 for tag in object.Viewfinder_Object_Properties.Tags.strip().split(
        #                     ","
        #                 )
        #             ]
        #             Obj_tags = list(dict.fromkeys(Obj_tags))
        #
        #             if any(t in Obj_tags for t in filter_tags):
        #                 tag_filter_objects.append(object)

        # objects = tag_filter_objects
        objects.sort(
            key=lambda x: x.Viewfinder_Object_Properties.priority, reverse=True
        )
        objects.sort(key=lambda x: x.Viewfinder_Object_Properties.Pin, reverse=True)

        return objects

    def draw_camera_list(self, context, layout, cameras):
        scn = context.scene
        scn_properties = scn.Viewfinder_Scene_Properties

        preferences = Utility_Functions.get_addon_preferences()
        if len(cameras) > 0:
            for object in cameras:
                box = layout.box()
                row = box.row(align=True)

                Object_Properties = object.Viewfinder_Object_Properties

                Show_Camera_Data = Utility_Functions.draw_subpanel(
                    Object_Properties,
                    Object_Properties.SHOW_Camera_Data_Options,
                    "SHOW_Camera_Data_Options",
                    "",
                    row,
                )

                if preferences.ICON_Camera_List_Reorder:
                    op = row.operator(
                        "viewfinder.reorder_cameras", text="", icon="TRIA_UP"
                    )
                    op.direction = "UP"
                    op.object = object.name
                    op = row.operator(
                        "viewfinder.reorder_cameras", text="", icon="TRIA_DOWN"
                    )
                    op.direction = "DOWN"
                    op.object = object.name

                if preferences.ICON_Camera_List_Priority:
                    split = row.split(factor=0.08, align=True)
                    split.prop(Object_Properties, "priority", text="")
                    row = split.row(align=True)

                if preferences.ICON_Camera_List_Pin:
                    row.prop(Object_Properties, "Pin", text="", icon="PINNED")

                if preferences.ICON_Camera_List_View_Camera:
                    is_viewing = False
                    # context.space_data.region_3d.view_perspective
                    if (
                        context.scene.camera == object
                        and context.space_data.region_3d.view_perspective == "CAMERA"
                    ):
                        is_viewing = True

                    if is_viewing:
                        operator = row.operator(
                            "viewfinder.view_camera",
                            text="",
                            icon="RESTRICT_VIEW_OFF",
                            depress=True,
                        )
                    else:
                        operator = row.operator(
                            "viewfinder.view_camera", text="", icon="RESTRICT_VIEW_ON"
                        )
                    operator.object = object.name

                if preferences.ICON_Camera_List_Select_Target:
                    col = row.column()

                    col.enabled = False

                    if object:
                        for constraint in object.constraints:
                            if hasattr(constraint, "target"):
                                target = constraint.target
                                if target:
                                    col.enabled = True

                    targets = []

                    for cons in object.constraints:
                        if hasattr(cons, "target"):
                            if constraint.target:
                                targets.append(constraint.target)

                    if len(targets) > 1:
                        operator = col.operator(
                            "viewfinder.list_select_target", text="", icon="EMPTY_DATA"
                        )
                        operator.object = object.name

                    else:
                        operator = col.operator(
                            "viewfinder.select_target", text="", icon="EMPTY_DATA"
                        )
                        operator.object = object.name

                if preferences.ICON_Camera_List_Select_Camera:
                    operator = row.operator(
                        "viewfinder.select_camera", text="", icon="RESTRICT_SELECT_OFF"
                    )
                    operator.object = object.name

                if preferences.ICON_Camera_List_Find_Camera:
                    operator = row.operator(
                        "viewfinder.find_camera", text="", icon="VIEWZOOM"
                    )
                    operator.object = object.name

                if context.scene.camera == object:
                    row.prop(object, "name", text="", icon="SCENE_DATA")
                else:
                    row.prop(object, "name", text="")

                if preferences.ICON_Camera_List_Tags:
                    row.prop(Object_Properties, "Tags", text="", icon="BOOKMARKS")

                # if scn_properties.Set_Resolution:
                #     if preferences.ICON_Camera_List_Resolution:
                #         if Object_Properties.Use_Resolution:
                #             row.prop(Object_Properties, "Resolution_X", text="")
                #             row.prop(Object_Properties, "Resolution_Y", text="")

                if preferences.ICON_Camera_List_Set_Marker:
                    operator = row.operator(
                        "viewfinder.add_camera_marker", text="", icon="MARKER_HLT"
                    )
                    operator.object = object.name

                if preferences.ICON_Camera_List_Duplicate_Camera:
                    operator = row.operator(
                        "viewfinder.duplicate_camera", text="", icon="DUPLICATE"
                    )
                    operator.object = object.name

                if preferences.ICON_Camera_List_Remove_Camera:
                    operator = row.operator(
                        "viewfinder.remove_camera", text="", icon="TRASH"
                    )
                    operator.object = object.name

                if Show_Camera_Data:
                    self.draw_camera_data(object, box)

                layout.separator()
        else:
            box = layout.box()
            box.label(text="No Camera Found", icon="INFO")
            layout.separator()

        row = layout.row(align=True)

        operator = row.operator(
            "viewfinder.create_camera_from_view",
            text="New Camera From View",
            icon="PLUS",
        )

        # operator = row.operator("viewfinder.add_aligned_camera", text="Camera", icon="ADD")
        # operator.side = "FRONT"
        # operator.View_Camera = False
        # operator.position = "CURSOR"
        # operator.Distance = 0

    def draw_camera_data(self, object, layout):
        Object_Properties = object.Viewfinder_Object_Properties

        col = layout.column(align=True)
        col.prop(object.data, "type")

        if object.data.type == "ORTHO":
            col.prop(object.data, "ortho_scale")

        if object.data.type in ["PERSP", "PANO"]:
            row = col.row(align=True)
            row.prop(object.data, "lens_unit", expand=True)

            if object.data.lens_unit == "MILLIMETERS":
                col.prop(object.data, "lens")
            if object.data.lens_unit == "FOV":
                col.prop(object.data, "angle")

        col.separator()
        row = col.row(align=True)
        row.prop(object.data, "shift_x")
        row.prop(object.data, "shift_y")

        row = col.row(align=True)
        row.prop(object.data, "clip_start")
        row.prop(object.data, "clip_end")

        col.separator()
        col.prop(Object_Properties, "Tags", text="Tags")
        col.separator()

        # col.prop(Object_Properties, "Use_Resolution", text="Use Resolution")
        # if Object_Properties.Use_Resolution:
        #     row2 = col.row(align=True)
        #     row2.prop(Object_Properties, "Resolution_X", text="Resolution X")
        #     row2.prop(Object_Properties, "Resolution_Y", text="Resolution Y")
        #     operator = row2.operator(
        #         "viewfinder.resolution_match_scene", text="", icon="SCENE"
        #     )
        #     operator.object = object.name
        # col.separator()

        if Utility_Functions.draw_subpanel(
            Object_Properties,
            Object_Properties.SHOW_Display_Options,
            "SHOW_Display_Options",
            "Display Option",
            col,
        ):
            col.prop(object, "show_in_front")
            col.separator()
            col.prop(object.data, "display_size")
            col.prop(object.data, "show_limits")
            col.prop(object.data, "show_sensor")
            col.prop(object.data, "show_name")
            col.prop(object.data, "show_passepartout")
            if object.data.show_passepartout:
                col.prop(object.data, "passepartout_alpha")

        if Utility_Functions.draw_subpanel(
            Object_Properties,
            Object_Properties.SHOW_Sensor_Options,
            "SHOW_Sensor_Options",
            "Sensor Fit",
            col,
        ):
            col.prop(object.data, "sensor_fit", text="")
            if object.data.sensor_fit in ["AUTO", "HORIZONTAL"]:
                col.prop(object.data, "sensor_width")
            if object.data.sensor_fit in ["VERTICAL"]:
                col.prop(object.data, "sensor_height")

        if Utility_Functions.draw_subpanel(
            Object_Properties,
            Object_Properties.SHOW_DOF_Options,
            "SHOW_DOF_Options",
            "Depth of Field",
            col,
        ):
            col.prop(object.data.dof, "use_dof")
            if object.data.dof.use_dof:
                col.prop(object.data.dof, "focus_object", text="")
                if not object.data.dof.focus_object:
                    col.prop(object.data.dof, "focus_distance")

                col.label(text="Aperture")

                col.prop(object.data.dof, "aperture_fstop")
                col.prop(object.data.dof, "aperture_blades")
                col.prop(object.data.dof, "aperture_rotation")
                col.prop(object.data.dof, "aperture_ratio")


classes = [VIEWFINDER_PT_Camera_List_Panel]


def register():
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)


if __name__ == "__main__":
    register()
